#include "util.h"
#include <math.h>
#include <stdio.h>
#include "image.h"


//helper function to show the results of an LK operation
void ShowResults(const IplImage * frame1,	//the first image
				 const IplImage * frame2,	//the secont image
				 CvMat * warp				//the warp between the two images
				 )
{
	CvSize sz;
	sz.height = frame1->height;
	sz.width = frame1->width;

	//allocate images
	IplImage *warped = cvCreateImage(sz, frame2->depth, frame2->nChannels);
	IplImage *result = cvCreateImage(sz, frame1->depth, frame1->nChannels);
	
	
	//calculate the warped and error images
	WarpIm(frame2,warp,warped);
	cvSub(frame1,warped,result);
	
	//show the images in sequence
	ShowImage(frame1, "frame1");
	ShowImage(frame2, "frame2");
	ShowImage(warped, "frame2warped");
	ShowImage(result,"result");

	cvReleaseImage( &warped ); 
	cvReleaseImage( &result ); 
}

//print a matrix in human-readable form
void PrintMatrix(const CvMat * mat	//the matrix to print
				 )
{
	for (int r = 0; r < mat->rows; r++)
	{
		for (int c = 0; c < mat->cols; c++)
			printf("%f\t", CV_MAT_ELEM(*mat,float,r,c));
		printf("\n");
	}
	printf("\n");
}

// Calculate the length of vector v
float length(const CvPoint2D32f& v)
{
	return sqrt(v.x*v.x + v.y*v.y);
}

// Apply affine matrix xform to pt, and return the result
CvPoint2D32f applyAffine(const CvMat * xform, CvPoint2D32f pt)
{
	CvMat *srcPos = cvCreateMat(3,1,CV_32F);
	CvMat *dstPos = cvCreateMat(2,1,CV_32F);

	//calculate the sub-pixel position of (x,y)_source in dest
	CV_MAT_ELEM(*srcPos,float,0,0) = (float)pt.x;
	CV_MAT_ELEM(*srcPos,float,1,0) = (float)pt.y;
	CV_MAT_ELEM(*srcPos,float,2,0) = 1.0f;
	cvmMul(xform,srcPos,dstPos);
	
	//these are the warped coordinates
	float spX = CV_MAT_ELEM(*dstPos,float,0,0);
	float spY = CV_MAT_ELEM(*dstPos,float,1,0);

	cvReleaseMat(&srcPos);
	cvReleaseMat(&dstPos);

	return cvPoint2D32f(spX, spY);
}
